/* fs/aio.c
 *	An async IO implementation for Linux
 *	Written by Benjamin LaHaise <bcrl@redhat.com>
 *
 *	Implements an efficient asynchronous io interface.
 *
 *	Copyright 2000, 2001, 2002 Red Hat, Inc.  All Rights Reserved.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along
 *   with this program; if not, write to the Free Software Foundation, Inc.,
 *   59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
//#define DEBUG 1

#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/file.h>
#include <linux/mm.h>
#include <linux/mman.h>
#include <linux/vmalloc.h>
#include <linux/iobuf.h>
#include <linux/slab.h>
#include <linux/timer.h>
#include <linux/brlock.h>
#include <linux/aio.h>
#include <linux/smp_lock.h>
#include <linux/compiler.h>
#include <linux/poll.h>
#include <linux/brlock.h>
#include <linux/module.h>

#include <asm/uaccess.h>
#include <asm/vsyscall.h>

#if DEBUG > 1
#define dprintk		printk
#else
#define dprintk(x...)	do { ; } while (0)
#endif

/*------ sysctl variables----*/
unsigned aio_nr;		/* current system wide number of aio requests */
unsigned aio_max_nr = 0x10000;	/* system wide maximum number of aio requests */
unsigned aio_max_size = 0x20000;	/* 128KB per chunk */
unsigned aio_max_pinned;		/* set to mem/4 in aio_setup */
/*----end sysctl variables---*/

static spinlock_t aio_read_lock = SPIN_LOCK_UNLOCKED;
static rwlock_t aio_req_lock = RW_LOCK_UNLOCKED;

static kmem_cache_t	*kiocb_cachep;
static kmem_cache_t	*kioctx_cachep;

/* tunable.  Needs to be added to sysctl. */
int max_aio_reqs = 0x10000;

/* Used for rare fput completion. */
static void aio_fput_routine(void *);
static struct tq_struct	fput_tqueue = {
	routine:	aio_fput_routine,
};

static spinlock_t	fput_lock = SPIN_LOCK_UNLOCKED;
LIST_HEAD(fput_head);

/* forward prototypes */
static void generic_aio_complete_read(void *_iocb, struct kvec *vec, ssize_t res);
static void generic_aio_complete_write(void *_iocb, struct kvec *vec, ssize_t res);

/* aio_setup
 *	Creates the slab caches used by the aio routines, panic on
 *	failure as this is done early during the boot sequence.
 */
static int __init aio_setup(void)
{
	kiocb_cachep = kmem_cache_create("kiocb", sizeof(struct kiocb),
				0, SLAB_HWCACHE_ALIGN, NULL, NULL);
	if (!kiocb_cachep)
		panic("unable to create kiocb cache\n");

	kioctx_cachep = kmem_cache_create("kioctx", sizeof(struct kioctx),
				0, SLAB_HWCACHE_ALIGN, NULL, NULL);
	if (!kioctx_cachep)
		panic("unable to create kioctx cache");

	aio_max_pinned = num_physpages/4;

	printk(KERN_NOTICE "aio_setup: num_physpages = %u\n", aio_max_pinned);
	printk(KERN_NOTICE "aio_setup: sizeof(struct page) = %d\n", (int)sizeof(struct page));

	return 0;
}

static void ioctx_free_reqs(struct kioctx *ctx)
{
	struct list_head *pos, *next;
	list_for_each_safe(pos, next, &ctx->free_reqs) {
		struct kiocb *iocb = list_kiocb(pos);
		list_del(&iocb->list);
		kmem_cache_free(kiocb_cachep, iocb);
	}
}

/* ioctx_alloc
 *	Allocates and initializes an ioctx.  Returns an ERR_PTR if it failed.
 */
static struct kioctx *ioctx_alloc(unsigned nr_reqs)
{
	struct kioctx *ctx;
	unsigned i;
	long size;

	/* Round off to a power of 2.  Needed for cheap mask operations */
	for (i=1; i<nr_reqs; i<<=1)
		;

	nr_reqs = i;

	/* Prevent overflows */
	if ((nr_reqs > (0x10000000U / sizeof(struct io_event))) ||
	    (nr_reqs > (0x10000000U / sizeof(struct kiocb)))) {
		pr_debug("ENOMEM: nr_reqs too high\n");
		return ERR_PTR(-EINVAL);
	}

	if (nr_reqs > aio_max_nr)
		return ERR_PTR(-EAGAIN);

	ctx = kmem_cache_alloc(kioctx_cachep, GFP_KERNEL);
	if (!ctx)
		return ERR_PTR(-ENOMEM);

	memset(ctx, 0, sizeof(*ctx));
	ctx->max_reqs = nr_reqs;
	ctx->mm = current->mm;
	atomic_inc(&ctx->mm->mm_count);

	atomic_set(&ctx->users, 1);
	spin_lock_init(&ctx->lock);
	spin_lock_init(&ctx->ring_lock);
	init_waitqueue_head(&ctx->wait);

	INIT_LIST_HEAD(&ctx->free_reqs);
	INIT_LIST_HEAD(&ctx->active_reqs);

	/* Allocate nr_reqs iocbs for io.  Free iocbs are on the 
	 * ctx->free_reqs list.  When active they migrate to the 
	 * active_reqs list.  During completion and cancellation 
	 * the request may temporarily not be on any list.
	 */
	for (i=0; i<nr_reqs; i++) {
		struct kiocb *iocb = kmem_cache_alloc(kiocb_cachep, GFP_KERNEL);
		if (!iocb)
			goto out_freereqs;
		memset(iocb, 0, sizeof(*iocb));
		iocb->key = i;
		list_add(&iocb->list, &ctx->free_reqs);
	}

	/* Compensate for the ring buffer's head/tail overlap entry */
	nr_reqs *= 2;

	size = sizeof(struct aio_ring);
	size += sizeof(struct io_event) * nr_reqs;

	/* Try to use alloc_pages first... */
	for (i=0; (PAGE_SIZE << i) < size; i++)
		;
	ctx->ring_order = i;
	ctx->ring = (void *)__get_free_pages(GFP_KERNEL, i);

	if (!ctx->ring) {
		/* vmalloc it is... */
		ctx->ring = vmalloc(size);
		if (!ctx->ring)
			goto out_freereqs;
		ctx->ring_was_vmallocd = 1;
	} else
		ctx->ring_was_vmallocd = 0;

	memset(ctx->ring, 0, size);
	ctx->ring_mask = nr_reqs - 1;		/* trusted copy */
	ctx->ring->mask = ctx->ring_mask;	/* user copy */

	/* now link into global list.  kludge.  FIXME */
	br_write_lock(BR_AIO_REQ_LOCK);			
	if (unlikely(aio_nr + ctx->max_reqs > aio_max_nr))
		goto out_cleanup;
	aio_nr += ctx->max_reqs;	/* undone by __put_ioctx */
	ctx->ring->id = ctx->user_id = ++current->mm->new_ioctx_id;
	ctx->next = current->mm->ioctx_list;
	current->mm->ioctx_list = ctx;
	br_write_unlock(BR_AIO_REQ_LOCK);

	dprintk("aio: allocated ioctx %p[%ld]: mm=%p mask=0x%x\n",
		ctx, ctx->user_id, current->mm, ctx->ring->mask);
	return ctx;

out_cleanup:
	br_write_unlock(BR_AIO_REQ_LOCK);
	ctx->max_reqs = 0;	/* prevent __put_ioctx from sub'ing aio_nr */
	__put_ioctx(ctx);
	return ERR_PTR(-EAGAIN);

out_freereqs:
	ioctx_free_reqs(ctx);
out_freectx:
	kmem_cache_free(kioctx_cachep, ctx);
	ctx = ERR_PTR(-ENOMEM);

	dprintk("aio: error allocating ioctx %p\n", ctx);
	return ctx;
}

/* exit_aio: called when the last user of mm goes away.  At this point, 
 * there is no way for any new requests to be submited or any of the 
 * io_* syscalls to be called on the context.  However, there may be 
 * outstanding requests which hold references to the context; as they 
 * go away, they will call put_ioctx and release any pinned memory
 * associated with the request (held via struct page * references).
 */
void exit_aio(struct mm_struct *mm)
{
	struct kioctx *ctx = mm->ioctx_list;
	mm->ioctx_list = NULL;
	while (ctx) {
		struct kioctx *next = ctx->next;
		ctx->next = NULL;
		/* TODO: cancel outstanding requests if possible */
		put_ioctx(ctx);
		ctx = next;
	}
}

/* __put_ioctx
 *	Called when the last user of an aio context has gone away,
 *	and the struct needs to be freed.
 */
void __put_ioctx(struct kioctx *ctx)
{
	unsigned nr_reqs = ctx->max_reqs;

	mmdrop(ctx->mm);
	ctx->mm = NULL;
	pr_debug("__put_ioctx: freeing %p\n", ctx);
	if (ctx->ring_was_vmallocd)
		vfree(ctx->ring);
	else
		free_pages((unsigned long)ctx->ring, ctx->ring_order);

	ioctx_free_reqs(ctx);
	kmem_cache_free(kioctx_cachep, ctx);

	br_write_lock(BR_AIO_REQ_LOCK);
	aio_nr -= nr_reqs;
	br_write_unlock(BR_AIO_REQ_LOCK);
}

/* aio_get_req
 *	Allocate a slot for an aio request.  Increments the users count
 * of the kioctx so that the kioctx stays around until all requests are
 * complete.  Returns -EAGAIN if no requests are free.
 */
static inline struct kiocb *__aio_get_req(struct kioctx *ctx)
{
	struct kiocb *req = NULL;

	/* Use cmpxchg instead of spin_lock? */
	spin_lock_irq(&ctx->lock);
	if (!list_empty(&ctx->free_reqs) &&
	    (ctx->reqs_active < aio_ring_avail(ctx->ring))) {
		req = list_kiocb(ctx->free_reqs.next);
		list_del(&req->list);
		list_add(&req->list, &ctx->active_reqs);
		ctx->reqs_active++;
		req->user_obj = NULL;
		get_ioctx(ctx);

		if (req->ctx)
			BUG();
		req->ctx = ctx;
	}
	spin_unlock_irq(&ctx->lock);

	return req;
}

static inline struct kiocb *aio_get_req(struct kioctx *ctx)
{
	struct kiocb *req;
	/* Handle a potential starvation case -- should be exceedingly rare as 
	 * requests will be stuck on fput_head only if the aio_fput_routine is 
	 * delayed and the requests were the last user of the struct file.
	 */
	req = __aio_get_req(ctx);
	if (unlikely(NULL == ctx)) {
		aio_fput_routine(NULL);
		req = __aio_get_req(ctx);
	}
	return req;
}

static inline void really_put_req(struct kioctx *ctx, struct kiocb *req)
{
	req->ctx = NULL;
	req->filp = NULL;
	req->user_obj = NULL;
	ctx->reqs_active--;
	list_add(&req->list, &ctx->free_reqs);
}

static void aio_fput_routine(void *data)
{
	spin_lock_irq(&fput_lock);
	while (likely(!list_empty(&fput_head))) {
		struct kiocb *req = list_kiocb(fput_head.next);
		struct kioctx *ctx = req->ctx;
		struct kiocb *next = req->user_obj;

		list_del(&req->list);
		spin_unlock_irq(&fput_lock);

		/* Complete the fput */
		__fput(req->filp);

		/* Link the iocb into the context's free list */
		spin_lock_irq(&ctx->lock);
		really_put_req(ctx, req);
		spin_unlock_irq(&ctx->lock);

		put_ioctx(ctx);
		spin_lock_irq(&fput_lock);
	}
	spin_unlock_irq(&fput_lock);
}

static void __aio_put_req(struct kioctx *ctx, struct kiocb *req)
{
	dprintk(KERN_DEBUG "aio_put(%p): f_count=%d\n",
		req, atomic_read(&req->filp->f_count));

	list_del(&req->list);		/* remove from active_reqs */
	req->cancel = NULL;

	/* Must be done under the lock to serialise against cancellation.
	 * Call this aio_fput as it duplicates fput via the fput_tqueue.
	 */
	if (unlikely(atomic_dec_and_test(&req->filp->f_count))) {
		get_ioctx(ctx);
		spin_lock(&fput_lock);
		list_add(&req->list, &fput_head);
		spin_unlock(&fput_lock);
		schedule_task(&fput_tqueue);
	} else
		really_put_req(ctx, req);
}

static void aio_put_req(struct kioctx *ctx, struct kiocb *req)
{
	spin_lock_irq(&ctx->lock);
	__aio_put_req(ctx, req);
	spin_unlock_irq(&ctx->lock);
	put_ioctx(ctx);
}

/*	Lookup an ioctx id.  ioctx_list is lockless for reads.
 *	FIXME: this is O(n) and is only suitable for development.
 */
static inline struct kioctx *lookup_ioctx(unsigned long ctx_id)
{
	struct kioctx *ioctx;
	struct mm_struct *mm;

	br_read_lock(BR_AIO_REQ_LOCK);
	mm = current->mm;
	for (ioctx = mm->ioctx_list; ioctx; ioctx = ioctx->next)
		if (likely(ioctx->user_id == ctx_id && !ioctx->dead)) {
			get_ioctx(ioctx);
			break;
		}
	br_read_unlock(BR_AIO_REQ_LOCK);

	return ioctx;
}

/* aio_complete
 *	Called when the io request on the given iocb is complete.
 */
void aio_complete(struct kiocb *iocb, long res, long res2)
{
	struct kioctx	*ctx = iocb->ctx;
	struct aio_ring	*ring = ctx->ring;
	struct io_event	*event;
	unsigned long	flags;
	unsigned long	tail;

	/* add a completion event to the ring buffer.
	 * must be done holding ctx->lock to prevent
	 * other code from messing with the tail
	 * pointer since we might be called from irq
	 * context.
	 */
	spin_lock_irqsave(&ctx->lock, flags);

	tail = ring->tail;
	event = &ring->io_events[tail];
	tail = (tail + 1) & ring->mask;

	event->obj = (u64)(unsigned long)iocb->user_obj;
	event->data = iocb->user_data;
	event->res = res;
	event->res2 = res2;

	dprintk("aio_complete: %p[%lu]: %p: %p %Lx %lx %lx\n",
		ctx, tail, iocb, iocb->user_obj, iocb->user_data, res, res2);

	/* after flagging the request as done, we
	 * must never even look at it again
	 */
	barrier();

	ring->tail = tail;

	wmb();
	if (!ring->woke)
		ring->woke = 1;

	pr_debug("added to ring %p at [%lu]\n", iocb, tail);

	/* everything turned out well, dispose of the aiocb. */
	__aio_put_req(ctx, iocb);

	spin_unlock_irqrestore(&ctx->lock, flags);
	put_ioctx(ctx);

	wake_up(&ctx->wait);
}

/* aio_read_evt
 *	Pull an event off of the ioctx's event ring.  Returns the number of 
 *	events fetched (0 or 1 ;-)
 *	FIXME: make this use cmpxchg.
 *	TODO: make the ringbuffer user mmap()able (requires FIXME).
 */
static int aio_read_evt(struct kioctx *ioctx, struct io_event *ent)
{
	struct aio_ring *ring = ioctx->ring;
	unsigned long head;
	int ret = 0;

	dprintk("in aio_read_evt h%lu t%lu m%lu\n",
		 (unsigned long)ring->head, (unsigned long)ring->tail,
		 (unsigned long)ring->mask);
	barrier();
	if (ring->head == ring->tail)
		goto out;

	spin_lock(&ioctx->ring_lock);

	head = ring->head;
	if (head != ring->tail) {
		*ent = ring->io_events[head];
		head = (head + 1) & ioctx->ring_mask;
		barrier();
		ring->head = head;
		ret = 1;
	}
	spin_unlock(&ioctx->ring_lock);

out:
	dprintk("leaving aio_read_evt: %d  h%lu t%lu\n", ret,
		 (unsigned long)ring->head, (unsigned long)ring->tail);
	return ret;
}

struct timeout {
	struct timer_list	timer;
	int			timed_out;
	wait_queue_head_t	wait;
};

static void timeout_func(unsigned long data)
{
	struct timeout *to = (struct timeout *)data;

	to->timed_out = 1;
	wake_up(&to->wait);
}

static inline void init_timeout(struct timeout *to)
{
	init_timer(&to->timer);
	to->timer.data = (unsigned long)to;
	to->timer.function = timeout_func;
	to->timed_out = 0;
	init_waitqueue_head(&to->wait);
}

static inline void set_timeout(struct timeout *to, const struct timespec *ts)
{
	unsigned long how_long;

	if (!ts->tv_sec && !ts->tv_nsec) {
		to->timed_out = 1;
		return;
	}

	how_long = ts->tv_sec * HZ;
#define HZ_NS (1000000000 / HZ)
	how_long += (ts->tv_nsec + HZ_NS - 1) / HZ_NS;
	
	to->timer.expires = jiffies + how_long;
	add_timer(&to->timer);
}

static inline void clear_timeout(struct timeout *to)
{
	del_timer_sync(&to->timer);
}

static int read_events(struct kioctx *ctx, int nr, struct io_event *event,
			const struct timespec *timeout)
{
	struct task_struct	*tsk = current;
	DECLARE_WAITQUEUE(wait, tsk);
	DECLARE_WAITQUEUE(to_wait, tsk);
	int			ret;
	int			i = 0;
	struct io_event		ent;
	struct timeout		to;

	/* needed to zero any padding within an entry (there shouldn't be 
	 * any, but C is fun!
	 */
	memset(&ent, 0, sizeof(ent));
	ret = 0;

	while (likely(i < nr)) {
		ret = aio_read_evt(ctx, &ent);
		if (unlikely(ret <= 0))
			break;

		dprintk("read event: %Lx %Lx %Lx %Lx\n",
			ent.data, ent.obj, ent.res, ent.res2);

		/* FIXME: split checks in two */
		ret = -EFAULT;
		if (unlikely(copy_to_user(event, &ent, sizeof(ent)))) {
			dprintk("aio: lost an event due to EFAULT.\n");
			break;
		}
		ret = 0;

		/* Good, event copied to userland, update counts. */
		event ++;
		i ++;
	}

	if (i)
		return i;
	if (ret)
		return ret;

	/* End fast path */

	init_timeout(&to);
	if (timeout) {
		struct timespec	ts;
		ret = -EFAULT;
		if (unlikely(copy_from_user(&ts, timeout, sizeof(ts))))
			goto out;

		set_timeout(&to, &ts);
		if (to.timed_out)
			timeout = 0;
	}

	while (likely(i < nr)) {
		add_wait_queue_exclusive_lifo(&ctx->wait, &wait);
		add_wait_queue(&to.wait, &to_wait);
		do {
			set_task_state(tsk, TASK_INTERRUPTIBLE);

			ret = aio_read_evt(ctx, &ent);
			if (ret)
				break;
			if (i)
				break;
			ret = 0;
			if (to.timed_out)	/* Only check after read evt */
				break;
			schedule();
			if (signal_pending(tsk)) {
				ret = -EINTR;
				break;
			}
			/*ret = aio_read_evt(ctx, &ent);*/
		} while (1) ;

		set_task_state(tsk, TASK_RUNNING);
		remove_wait_queue(&ctx->wait, &wait);
		remove_wait_queue(&to.wait, &to_wait);

		if (unlikely(ret <= 0))
			break;

		ret = -EFAULT;
		if (unlikely(copy_to_user(event, &ent, sizeof(ent)))) {
			dprintk("aio: lost an event due to EFAULT.\n");
			break;
		}

		/* Good, event copied to userland, update counts. */
		event ++;
		i ++;
	}

	if (timeout)
		clear_timeout(&to);
out:
	return i ? i : ret;
}

/* Take an ioctx and remove it from the list of ioctx's.  Protects 
 * against races with itself via ->dead.
 */
static void io_destroy(struct kioctx *ioctx)
{
	struct kioctx **tmp;
	int was_dead;

	/* delete the entry from the list is someone else hasn't already */
	br_write_lock(BR_AIO_REQ_LOCK);
	was_dead = ioctx->dead;
	ioctx->dead = 1;
	for (tmp = &current->mm->ioctx_list; *tmp && *tmp != ioctx;
	     tmp = &(*tmp)->next)
		;
	if (*tmp)
		*tmp = ioctx->next;
	br_write_unlock(BR_AIO_REQ_LOCK);

	dprintk("aio_release(%p)\n", ioctx);
	put_ioctx(ioctx);	/* once for the lookup */
	if (likely(!was_dead))
		put_ioctx(ioctx);	/* twice for the list */
}

asmlinkage long sys_io_setup(unsigned nr_reqs, aio_context_t *ctxp)
{
	struct kioctx *ioctx = NULL;
	unsigned long ctx;
	long ret;

	ret = get_user(ctx, ctxp);
	if (unlikely(ret))
		goto out;

	ret = -EINVAL;
	if (unlikely(ctx || !nr_reqs || (int)nr_reqs < 0)) {
		pr_debug("EINVAL: io_setup: ctx or nr_reqs > max\n");
		goto out;
	}

	ret = -EAGAIN;
	if (unlikely(nr_reqs > max_aio_reqs))
		goto out;

	ioctx = ioctx_alloc(nr_reqs);
	ret = PTR_ERR(ioctx);
	if (!IS_ERR(ioctx)) {
		ret = put_user(ioctx->user_id, ctxp);
		if (!ret)
			return 0;
		io_destroy(ioctx);
	}

out:
	return ret;
}

/* aio_release
 *	Release the kioctx associated with the userspace handle.
 */
asmlinkage long sys_io_destroy(aio_context_t ctx)
{
	struct kioctx *ioctx = lookup_ioctx(ctx);
	if (likely(NULL != ioctx)) {
		io_destroy(ioctx);
		return 0;
	}
	pr_debug("EINVAL: io_destroy: invalid context id\n");
	return -EINVAL;
}

int generic_aio_poll(struct file *file, struct kiocb *req, struct iocb iocb)
{
	unsigned events = iocb.aio_buf;

	/* Did the user set any bits they weren't supposed to? (The 
	 * above is actually a cast.
	 */
	if (unlikely(events != iocb.aio_buf))
		return -EINVAL;
	
	return async_poll(req, events);
}

/* sys_io_submit
 *	Copy an aiocb from userspace into kernel space, then convert it to
 *	a kiocb, submit and repeat until done.  Error codes on copy/submit
 *	only get returned for the first aiocb copied as otherwise the size
 *	of aiocbs copied is returned (standard write sematics).
 */
asmlinkage long sys_io_submit(aio_context_t ctx_id, long nr, struct iocb **iocbpp)
{
	struct kioctx *ctx;
	long ret = 0;
	int i;

	if (unlikely(nr < 0))
		return -EINVAL;

	ctx = lookup_ioctx(ctx_id);
	if (unlikely(!ctx)) {
		pr_debug("EINVAL: io_submit: invalid context id\n");
		return -EINVAL;
	}

	for (i=0; i<nr; i++) {
		int (*op)(struct file *, struct kiocb *, struct iocb);
		struct iocb *iocbp, tmp;
		struct kiocb *req;
		struct file *file;

		ret = get_user(iocbp, iocbpp + i);
		if (unlikely(ret))
			break;

		ret = -EFAULT;
		if (copy_from_user(&tmp, iocbp, sizeof(tmp)))
			break;

		ret = -EINVAL;
		/* enforce forwards compatibility on users */
		if (unlikely(tmp.aio_reserved1 || tmp.aio_reserved2 || tmp.aio_reserved3)) {
			pr_debug("EINVAL: io_submit: reserve field set\n");
			break;
		}

		/* prevent overflows */
		if (unlikely(
		    (tmp.aio_buf != (unsigned long)tmp.aio_buf) ||
		    (tmp.aio_nbytes != (size_t)tmp.aio_nbytes) ||
		    ((ssize_t)tmp.aio_nbytes < 0)
		   )) {
			pr_debug("EINVAL: io_submit: overflow check\n");
			break;
		}

		file = fget(tmp.aio_fildes);
		ret = -EBADF;
		if (unlikely(!file))
			break;

		req = aio_get_req(ctx);
		ret = -EAGAIN;
		if (unlikely(!req)) {
			fput(file);
			break;
		}

		req->filp = file;
		tmp.aio_key = req->key;
		ret = put_user(tmp.aio_key, &iocbp->aio_key);
		if (unlikely(ret)) {
			dprintk("EFAULT: aio_key\n");
			goto out_put_req;
		}

		req->user_obj = iocbp;
		req->user_data = tmp.aio_data;
		req->buf = tmp.aio_buf;
		req->pos = tmp.aio_offset;
		req->size = tmp.aio_nbytes;

		switch (tmp.aio_lio_opcode) {
		case IOCB_CMD_PREAD:
			op = file->f_op->aio_read;
			ret = -EBADF;
			if (!(file->f_mode & FMODE_READ))
				goto out_put_req;
			break;
		case IOCB_CMD_PREADX:
			op = file->f_op->aio_readx;
			ret = -EBADF;
			if (!(file->f_mode & FMODE_READ))
				goto out_put_req;
			break;
		case IOCB_CMD_PWRITE:
			op = file->f_op->aio_write;
			ret = -EBADF;
			if (!(file->f_mode & FMODE_WRITE))
				goto out_put_req;
			break;
		case IOCB_CMD_FSYNC:
			op = file->f_op->aio_fsync;
			break;
		//case IOCB_CMD_POLL:
		//	op = generic_aio_poll;
		//	break;
		default:
			op = NULL;
			break;
		}

		ret = -EINVAL;
		if (unlikely(!op)) {
			pr_debug("EINVAL: io_submit: no operation provided\n");
			goto out_put_req;
		}

		ret = op(file, req, tmp);
		if (likely(!ret))
			continue;

		pr_debug("io_submit: op returned %ld\n", ret);
		aio_complete(req, ret, 0);
		ret = 0;	/* A completion event was sent, so 
				 * submit is a success. */
		continue;

	out_put_req:
		aio_put_req(ctx, req);
		break;
	}

	put_ioctx(ctx);
	run_task_queue(&tq_disk);
	return i ? i : ret;
}

static void generic_aio_next_chunk(void *_iocb)
{
	int (*kvec_op)(struct file *, kvec_cb_t, size_t, loff_t);
	struct kiocb *iocb = _iocb;
	int rw = iocb->this_size;
	unsigned long buf = iocb->buf;
	kvec_cb_t cb;
	ssize_t res;

	iocb->this_size = iocb->size - iocb->nr_transferred;
	if (iocb->this_size > aio_max_size)
		iocb->this_size = aio_max_size;

	buf += iocb->nr_transferred;
	cb.vec = mm_map_user_kvec(iocb->ctx->mm, rw, buf, iocb->this_size);
	cb.fn = (rw == READ) ? generic_aio_complete_read
			     : generic_aio_complete_write;
	cb.data = iocb;

	dprintk("generic_aio_rw: cb.vec=%p\n", cb.vec);
	if (unlikely(IS_ERR(cb.vec)))
		goto done;

	kvec_op = (rw == READ) ? iocb->filp->f_op->kvec_read
			       : iocb->filp->f_op->kvec_write;
	dprintk("submit: %d %d %d\n", iocb->this_size, iocb->nr_transferred, iocb->size);
	res = kvec_op(iocb->filp, cb, iocb->this_size,
		      iocb->pos + iocb->nr_transferred);
	if (!res) {
		dprintk("submit okay\n");
		return;
	}
	dprintk("submit failed: %d\n", res);
	
	cb.fn(cb.data, cb.vec, res);
	return;

done:
	if (!iocb->nr_transferred)
		BUG();
	aio_complete(iocb, iocb->nr_transferred, 0);
}

static void generic_aio_complete_rw(int rw, void *_iocb, struct kvec *vec, ssize_t res)
{
	struct kiocb *iocb = _iocb;

	unmap_kvec(vec, rw == READ);
	free_kvec(vec);

	if (res > 0)
		iocb->nr_transferred += res;

	/* Was this chunk successful?  Is there more left to transfer? */
	if (res == iocb->this_size && iocb->nr_transferred < iocb->size) {
		/* We may be in irq context, so queue processing in 
		 * process context.
		 */
		iocb->this_size = rw;
		INIT_TQUEUE(&iocb->tq, generic_aio_next_chunk, iocb);
		schedule_task(&iocb->tq);
		return;
	}

	aio_complete(iocb, iocb->nr_transferred ? iocb->nr_transferred : res,
		     0);
}

static void generic_aio_complete_read(void *_iocb, struct kvec *vec, ssize_t res)
{
	generic_aio_complete_rw(READ, _iocb, vec, res);
}

static void generic_aio_complete_write(void *_iocb, struct kvec *vec, ssize_t res)
{
	generic_aio_complete_rw(WRITE, _iocb, vec, res);
}

static ssize_t generic_aio_rw(int rw, struct file *file, struct kiocb *req, struct iocb iocb, size_t min_size)
{
	int (*kvec_op)(struct file *, kvec_cb_t, size_t, loff_t);
	unsigned long buf = iocb.aio_buf;
	size_t size = iocb.aio_nbytes;
	size_t	nr_read = 0;
	loff_t pos = iocb.aio_offset;
	kvec_cb_t cb;
	ssize_t res;

	if (rw == READ && likely(NULL != file->f_op->new_read)) {
		nr_read = file->f_op->new_read(file, (void *)buf, size,
					       &pos, F_ATOMIC);
		dprintk("from new_read: nr_read: %ld\n", (long)nr_read);
		if ((-EAGAIN == nr_read) || (-EWOULDBLOCKIO == nr_read))
			nr_read = 0;
		else if ((nr_read >= min_size) || (nr_read < 0)) {
			dprintk("returning nr_read: %ld\n", (long)nr_read);
			return nr_read;
		}
	}
	dprintk("nr_read: %ld\n", (long)nr_read);

	req->nr_transferred = nr_read;
	size -= nr_read;
	if (size > aio_max_size)
		/* We have to split up the request.  Pin the mm
		 * struct for further use with map_user_kvec later.
		 */
		size = aio_max_size;
	else
		req->buf = 0;

	req->this_size = size;

	buf += nr_read;
	cb.vec = map_user_kvec(rw, buf, size);
	cb.fn = (rw == READ) ? generic_aio_complete_read
			     : generic_aio_complete_write;
	cb.data = req;

	dprintk("generic_aio_rw: cb.vec=%p\n", cb.vec);
	if (IS_ERR(cb.vec))
		return nr_read ? nr_read : PTR_ERR(cb.vec);

	kvec_op = (rw == READ) ? file->f_op->kvec_read : file->f_op->kvec_write;

	res = kvec_op(file, cb, size, pos);
	if (unlikely(res != 0)) {
		/* If the first chunk was successful, we have to run
		 * the callback to attempt the rest of the io.
		 */
		if (res == size && req->buf) {
			cb.fn(cb.data, cb.vec, res);
			return 0;
		}

		unmap_kvec(cb.vec, rw == READ);
		free_kvec(cb.vec);
		if (nr_read) {
			if (res < 0)
				res = 0;
			res += nr_read;
		}
	}
	return res;
}

ssize_t generic_file_aio_read(struct file *file, struct kiocb *req, struct iocb iocb)
{
	return generic_aio_rw(READ, file, req, iocb, iocb.aio_nbytes);	
}

ssize_t generic_sock_aio_read(struct file *file, struct kiocb *req, struct iocb iocb)
{
	return generic_aio_rw(READ, file, req, iocb, 1);	
}

ssize_t generic_aio_write(struct file *file, struct kiocb *req, struct iocb iocb, size_t min_size)
{
	return generic_aio_rw(WRITE, file, req, iocb, min_size);
#if 0
	unsigned long buf = iocb.aio_buf;
	size_t size = iocb.aio_nbytes;
	loff_t pos = iocb.aio_offset;
	ssize_t	nr_written = 0;
	kvec_cb_t cb;
	long res;

	if (likely(NULL != file->f_op->new_write)) {
		nr_written = file->f_op->new_write(file, (void *)buf, size,
					       &pos, F_ATOMIC);
		pr_debug("generic_aio_write: new_write: %ld\n", (long)nr_written);
		if (-EAGAIN == nr_written)
			nr_written = 0;
		if ((nr_written >= min_size) || (nr_written < 0))
			return nr_written;
	}

	req->nr_transferred = nr_written;
	size -= nr_written;
	if (size > aio_max_size)
		size = aio_max_size;
	req->this_size = size;
	buf += nr_written;
	cb.vec = map_user_kvec(WRITE, buf, size);
	cb.fn = generic_aio_complete_write;
	cb.data = req;

	if (IS_ERR(cb.vec)) {
		pr_debug("generic_aio_write: map_user_kvec: %ld\n", PTR_ERR(cb.vec));
		return nr_written ? nr_written : PTR_ERR(cb.vec);
	}

	res = file->f_op->kvec_write(file, cb, size, iocb.aio_offset);
	pr_debug("generic_aio_write: kvec_write: %ld\n", res);
	if (unlikely(res != 0)) {
		unmap_kvec(cb.vec, 0);
		free_kvec(cb.vec);
		if (nr_written) {
			if (res < 0)
				res = 0;
			res += nr_written;
		}
	}
	return res;
#endif
}

ssize_t generic_file_aio_write(struct file *file, struct kiocb *req, struct iocb iocb)
{
	return generic_aio_write(file, req, iocb, iocb.aio_nbytes);	
}

struct kiocb *lookup_kiocb(struct kioctx *ctx, struct iocb *iocb, u32 key)
{
	/* todo. */
	return NULL;
}

asmlinkage long sys_io_cancel(aio_context_t ctx_id, struct iocb *iocb)
{
	struct kioctx *ctx;
	struct kiocb *kiocb;
	u64 user_data;
	u32 key;
	int ret;

	ret = get_user(key, &iocb->aio_key);
	if (unlikely(ret))
		return ret;

	ctx = lookup_ioctx(ctx_id);
	if (unlikely(!ctx))
		return -EINVAL;

	spin_lock_irq(&ctx->lock);
	ret = -EAGAIN;
	kiocb = lookup_kiocb(ctx, iocb, key);
	if (kiocb && kiocb->cancel)
		ret = kiocb->cancel(ctx, kiocb);
	spin_unlock_irq(&ctx->lock);

	put_ioctx(ctx);

	return ret;
}

asmlinkage long sys_io_wait(aio_context_t ctx_id, struct iocb *iocb, const struct timespec *timeout)
{
#if 0	/* FIXME.  later. */
	struct kioctx *ioctx;
	long ret = -EINVAL;
	unsigned key;
	long obj = (long)iocb;

	ioctx = lookup_ioctx(ctx_id);
	if (!ioctx)
		goto out;

	ret = get_user(key, &iocb->aio_key);
	if (ret)
		goto out;

	ret = __aio_complete(ioctx, key, obj, !!timeout);
	put_ioctx(ioctx);

out:
	return ret;
#endif
	return -ENOSYS;
}

asmlinkage long sys_io_getevents(aio_context_t ctx_id,
				   long nr,
				   struct io_event *events,
				   const struct timespec *timeout)
{
	struct kioctx *ioctx = lookup_ioctx(ctx_id);
	long ret = -EINVAL;

	if (likely(NULL != ioctx)) {
		ret = read_events(ioctx, nr, events, timeout);
		put_ioctx(ioctx);
	}

	return ret;
}

__initcall(aio_setup);

add_dynamic_syscall(sys_io_setup);
add_dynamic_syscall(sys_io_destroy);
add_dynamic_syscall(sys_io_submit);
add_dynamic_syscall(sys_io_cancel);
add_dynamic_syscall(sys_io_wait);
add_dynamic_syscall(sys_io_getevents);
EXPORT_SYMBOL_GPL(generic_file_kvec_read);
EXPORT_SYMBOL_GPL(generic_file_aio_read);
EXPORT_SYMBOL_GPL(generic_file_kvec_write);
EXPORT_SYMBOL_GPL(generic_file_aio_write);
EXPORT_SYMBOL_GPL(generic_file_new_read);
EXPORT_SYMBOL_GPL(aio_max_size);
EXPORT_SYMBOL_GPL(aio_complete);
