/*
 * linux/fs/nfs/direct.c
 *
 * High-performance direct I/O for the NFS client
 *
 * When an application requests uncached I/O, all read and write requests
 * are made directly to the server; data stored or fetched via these
 * requests is not cached in the Linux page cache.  The client does not
 * correct unaligned requests from applications.  All requested bytes are
 * held on permanent storage before a direct write system call returns to
 * an application.  Applications that manage their own data caching, such
 * as databases, make very good use of direct I/O on local file systems.
 *
 * Solaris implements an uncached I/O facility called directio() that
 * is used for backups and sequential I/O to very large files.  Solaris
 * also supports uncaching whole NFS partitions with "-o forcedirectio,"
 * an undocumented mount option.
 *
 * Note that I/O to read in executables (e.g. kernel_read) cannot use
 * direct (kiobuf) reads because there is no vma backing the passed-in
 * data buffer.
 *
 * Designed by Jeff Kimmel, Chuck Lever, and Trond Myklebust.
 *
 * Initial implementation:	12/2001 by Chuck Lever <cel@netapp.com>
 * RHAS 2.1 backport:		09/2003 by Chuck Lever <cel@netapp.com>
 */

#include <linux/config.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/file.h>
#include <linux/errno.h>
#include <linux/nfs_fs.h>
#include <linux/smp_lock.h>
#include <linux/sunrpc/clnt.h>
#include <linux/iobuf.h>

#include <asm/system.h>
#include <asm/uaccess.h>

#define NFSDBG_FACILITY		(NFSDBG_PAGECACHE | NFSDBG_VFS)
#define VERF_SIZE		(2 * sizeof(__u32))

#define NFS_DIRECT_DEBUG	0

/* We can't exceed (MAX_IOVEC - 2) pages per RPC, so cap the number
 * of pages at 4 for now */
#define NFS_MAX_DIRECT_IO_SIZE	(PAGE_SIZE * 4U)

static inline int
nfs_direct_read_rpc(struct file *file, struct nfs_readargs *arg)
{
	int result;
	struct inode * inode = file->f_dentry->d_inode;
	struct nfs_fattr fattr;
	struct rpc_message msg;
	struct nfs_readres res = {
		.fattr	= &fattr,
		.count	= arg->count,
		.eof	= 0,
	};

#if NFS_DIRECT_DEBUG
	int i;
	printk(KERN_ERR "%s: count=%d, offset=%Lu\n", __FUNCTION__,
			arg->count, arg->offset);
	for (i = 0; i < arg->nriov; i++)
		printk(KERN_ERR "%s: arg->iov[%d]: base=%p, len=%u\n",
				__FUNCTION__, i, arg->iov[i].iov_base,
					arg->iov[i].iov_len);
#endif

#ifdef CONFIG_NFS_V3
	msg.rpc_proc = (NFS_PROTO(inode)->version == 3) ?
						NFS3PROC_READ : NFSPROC_READ;
#else
	msg.rpc_proc = NFSPROC_READ;
#endif
	msg.rpc_argp = arg;
	msg.rpc_resp = &res;

	lock_kernel();
	msg.rpc_cred = nfs_file_cred(file);
	fattr.valid = 0;
	result = rpc_call_sync(NFS_CLIENT(inode), &msg, 0);
	nfs_refresh_inode(inode, &fattr);
	unlock_kernel();

	return result;
}

static inline int
nfs_direct_write_rpc(struct file *file, struct nfs_writeargs *arg,
	struct nfs_writeverf *verf)
{
	int result;
	struct inode *inode = file->f_dentry->d_inode;
	struct nfs_fattr fattr;
	struct rpc_message msg;
	struct nfs_writeres res = {
		.fattr	= &fattr,
		.verf	= verf,
		.count	= 0,
	};

#if NFS_DIRECT_DEBUG
	int i;
	printk(KERN_ERR "%s: count=%d, offset=%Lu\n", __FUNCTION__,
			arg->count, arg->offset);
	for (i = 0; i < arg->nriov; i++)
		printk(KERN_ERR "%s: arg->iov[%d]: base=%p, len=%u\n",
				__FUNCTION__, i, arg->iov[i].iov_base,
					arg->iov[i].iov_len);
#endif

#ifdef CONFIG_NFS_V3
	msg.rpc_proc = (NFS_PROTO(inode)->version == 3) ?
						NFS3PROC_WRITE : NFSPROC_WRITE;
#else
	msg.rpc_proc = NFSPROC_WRITE;
#endif
	msg.rpc_argp = arg;
	msg.rpc_resp = &res;

	lock_kernel();
	msg.rpc_cred = get_rpccred(nfs_file_cred(file));
	fattr.valid = 0;
	result = rpc_call_sync(NFS_CLIENT(inode), &msg, 0);
	nfs_refresh_inode(inode, &fattr);
	put_rpccred(msg.rpc_cred);
	unlock_kernel();

#ifdef CONFIG_NFS_V3
	if (NFS_PROTO(inode)->version == 3) {
		if (result > 0) {
			if ((arg->stable == NFS_FILE_SYNC) &&
			    (verf->committed != NFS_FILE_SYNC)) {
				printk(KERN_ERR
				"%s: server didn't sync stable write request\n",
					__FUNCTION__);
				return -EIO;
			}

			if (result != arg->count) {
				printk(KERN_INFO
				"%s: short write, count=%u, result=%d\n",
					__FUNCTION__, arg->count, result);
			}
		}
		return result;
	} else {
#endif
		verf->committed = NFS_FILE_SYNC; /* NFSv2 always syncs data */
		if (result == 0)
			return arg->count;
		return result;
#ifdef CONFIG_NFS_V3
	}
#endif
}

#ifdef CONFIG_NFS_V3
static inline int
nfs_direct_commit_rpc(struct inode *inode, loff_t offset, size_t count,
	struct nfs_writeverf *verf)
{
	int result;
	struct nfs_fattr fattr;
	struct nfs_writeargs	arg = {
		.fh	= NFS_FH(inode),
		.offset	= offset,
		.count	= count,
	};
	struct nfs_writeres	res = {
		.fattr	= &fattr,
		.verf	= verf,
		.count	= 0,
	};
	struct rpc_message	msg = {
		.rpc_proc	= NFS3PROC_COMMIT,
		.rpc_argp	= &arg,
		.rpc_resp	= &res,
		.rpc_cred	= NULL,
	};

	fattr.valid = 0;

	lock_kernel();
	result = rpc_call_sync(NFS_CLIENT(inode), &msg, 0);
	nfs_refresh_inode(inode, &fattr);
	unlock_kernel();

	return result;
}
#else
static inline int
nfs_direct_commit_rpc(struct inode *inode, loff_t offset, size_t count,
	struct nfs_writeverf *verf)
{
	return 0;
}
#endif

/*
 * Walk through the iobuf and create an iovec for each "rsize" bytes.
 */
static int
nfs_kiobuf_read(struct file *file, struct kiobuf_nfs_direct *iobuf, loff_t offset,
	size_t count)
{
	int curpage, total;
	struct inode *inode = file->f_dentry->d_inode;
	int rsize = NFS_SERVER(inode)->rsize;
	struct nfs_readargs args = {
		.fh	= NFS_FH(inode),
	};

	total = 0;
	curpage = 0;
	while (count) {
		int starting_offset, request, result, first, last, i;
		struct iovec *iovec = args.iov;

		request = count;
		if (count > rsize)
			request = rsize;
		if (count > NFS_MAX_DIRECT_IO_SIZE)
			request = NFS_MAX_DIRECT_IO_SIZE;
		args.count = request;
		args.offset = offset;
		args.nriov = 0;

		starting_offset = iobuf->offset;
		first = last = curpage;
		do {
			struct page *page = iobuf->maplist[curpage];

			if (curpage >= iobuf->nr_pages || !page)
				return -EFAULT;

			args.nriov++;
			iovec->iov_base = kmap(page) + starting_offset;
			iovec->iov_len = (PAGE_SIZE - starting_offset);
			if ((starting_offset + request) < PAGE_SIZE)
				iovec->iov_len = request;

			request -= iovec->iov_len;
			last = curpage;

			if (!request && (starting_offset + iovec->iov_len < PAGE_SIZE))
				break;

			starting_offset = 0;	/* zero after the first page */
			curpage++;
			iovec++;
		} while (request != 0);

		result = nfs_direct_read_rpc(file, &args);

		for (i = first; i <= last; i++) {
			flush_dcache_page(iobuf->maplist[i]);
			kunmap(iobuf->maplist[i]);
		}

		if (result <= 0) {
			if (!total) {
				if (result == -EISDIR)
					total = -EINVAL;
				else
					total = result;
			}
			break;
		}

		total += result;
		count -= result;
		offset += result;

		if (result < args.count)	/* NFSv2ism */
			break;
	};
	return total;
}

/*
 * Walk through the iobuf and create an iovec for each "wsize" bytes.
 * If only one network write is necessary, or if the O_SYNC flag or
 * 'sync' mount option are present, or if this is a V2 inode, use
 * FILE_SYNC.  Otherwise, use UNSTABLE and finish with a COMMIT.
 *
 * The mechanics of this function are much the same as nfs_direct_read,
 * with the added complexity of committing unstable writes.
 */
static int
nfs_kiobuf_write(struct file *file, struct kiobuf_nfs_direct *iobuf,
	loff_t offset, size_t count)
{
	int curpage, total;
	int need_commit = 0;
	loff_t save_offset = offset;
	struct inode *inode = file->f_dentry->d_inode;
	int wsize = NFS_SERVER(inode)->wsize;
	struct nfs_writeverf first_verf, ret_verf;
	struct nfs_writeargs args = {
		.fh	= NFS_FH(inode),
		.stable	= NFS_FILE_SYNC,
	};

#ifdef CONFIG_NFS_V3
	if ((NFS_PROTO(inode)->version == 3) && (count > wsize) &&
							(!IS_SYNC(inode)))
		args.stable = NFS_UNSTABLE;
#endif

retry:
	total = 0;
	curpage = 0;
	while (count) {
		int starting_offset, request, result, first, last, i;
		struct iovec *iovec = args.iov;

		request = count;
		if (count > wsize)
			request = wsize;
		if (count > NFS_MAX_DIRECT_IO_SIZE)
			request = NFS_MAX_DIRECT_IO_SIZE;
		args.count = request;
		args.offset = offset;
		args.nriov = 0;

		starting_offset = iobuf->offset;
		first = last = curpage;
		do {
			struct page *page = iobuf->maplist[curpage];

			if (curpage >= iobuf->nr_pages || !page)
				return -EFAULT;

			args.nriov++;
			iovec->iov_base = kmap(page) + starting_offset;
			iovec->iov_len = (PAGE_SIZE - starting_offset);
			if ((starting_offset + request) < PAGE_SIZE)
				iovec->iov_len = request;

			request -= iovec->iov_len;
			last = curpage;

			if (!request && (starting_offset + iovec->iov_len < PAGE_SIZE))
				break;

			starting_offset = 0;	/* zero after the first page */
			curpage++;
			iovec++;
		} while (request != 0);

		result = nfs_direct_write_rpc(file, &args, &ret_verf);

		for (i = first; i <= last; i++) {
			kunmap(iobuf->maplist[i]);
		}

		if (result <= 0) {
			if (!total)
				total = result;
			break;
		}

		if (!total)
			memcpy(&first_verf.verifier, &ret_verf.verifier,
								VERF_SIZE);
		if (ret_verf.committed != NFS_FILE_SYNC) {
			need_commit = 1;
			if (memcmp(&first_verf.verifier, &ret_verf.verifier,
								VERF_SIZE))
				goto print_retry;
		}

		total += result;
		count -= result;
		offset += result;
	};

	/*
	 * Commit data written so far, even in the event of an error
	 */
	if (need_commit) {
		if (nfs_direct_commit_rpc(inode, save_offset,
					iobuf->length - count, &ret_verf))
			goto print_retry;
		if (memcmp(&first_verf.verifier, &ret_verf.verifier,
								VERF_SIZE))
			goto print_retry;
	}

	return total;

print_retry:
	printk(KERN_INFO
		"%s: detected server restart; retrying with FILE_SYNC\n",
			__FUNCTION__);
	args.stable = NFS_FILE_SYNC;
	offset = save_offset;
	count = iobuf->length;
	goto retry;
}

/*
 * The following functions are an adapter layer that provides the same
 * function as generic_file_direct_IO and friends from later kernels.
 */

/* how much buffer data one iobuf can represent.  keep the maplist
 * array small, but try to stay efficient. */
#define NFS_KIOBUF_SIZE	(4096 << 10)	/* one page of 32-bit addresses */

/*
 * Mark all of the pages in a kiobuf as dirty
 *
 * This is the same as mark_dirty_kiobuf, but we avoid adding an EXPORT
 * and breaking ABI compatibility by including our own copy here.
 */
static void
nfs_mark_dirty_kiobuf(struct kiobuf_nfs_direct *iobuf, int bytes)
{
	int index, offset, remaining;
	struct page *page;
	
	index = iobuf->offset >> PAGE_SHIFT;
	offset = iobuf->offset & ~PAGE_MASK;
	remaining = bytes;
	if (remaining > iobuf->length)
		remaining = iobuf->length;
	
	while (remaining > 0 && index < iobuf->nr_pages) {
		page = iobuf->maplist[index];
		
		if (!PageReserved(page))
			SetPageDirty(page);

		remaining -= (PAGE_SIZE - offset);
		offset = 0;
		index++;
	}
}

/*
 * Convert these regular-looking parameters into a struct kiobuf suitable
 * for NFS direct I/O.
 */
ssize_t
nfs_direct_file_read(struct file *file, char *buf, size_t count,
		loff_t *ppos)
{
	ssize_t result;
	int iosize, progress;
	loff_t offset = *ppos;
	struct dentry *dentry = file->f_dentry;
	struct inode *inode = dentry->d_inode;
	struct kiobuf_nfs_direct *iobuf;
	loff_t size;

	if (!count)
		return 0;

	iobuf = kmalloc(sizeof(struct kiobuf_nfs_direct), GFP_KERNEL);
	if (!iobuf)
		return -ENOMEM;
	memset(iobuf, 0, sizeof(struct kiobuf_nfs_direct));

	filemap_fdatasync(inode->i_mapping);
	down(&inode->i_sem);
	result = nfs_wb_all(inode);
	up(&inode->i_sem);
	filemap_fdatawait(inode->i_mapping);
	if (result < 0) {
		goto out_free;
	}

	result = 0;
	size = inode->i_size;
	if (*ppos > size)
		goto out_free;
	if (offset + count > size)
		count = size - offset;

	dfprintk(VFS, "NFS: direct_IO(READ) (%s/%s) off/cnt(%Lu/%d)\n",
			dentry->d_parent->d_name.name, dentry->d_name.name,
			offset, count);

	progress = 0;
	while (count > 0) {
		iosize = count;
		if (iosize > NFS_KIOBUF_SIZE)
			iosize = NFS_KIOBUF_SIZE;

		result = map_user_kiobuf(READ, (struct kiobuf *)iobuf, (unsigned long) buf,
				iosize);
		if (result)
			break;

		result = nfs_kiobuf_read(file, iobuf, offset + progress, count);
		if (result > 0) {
			nfs_mark_dirty_kiobuf(iobuf, result);
			count -= result;
			buf += result;
			progress += result;
		}

		unmap_kiobuf(iobuf);
		kfree(iobuf->maplist);

		if (result != iosize)
			break;
	}

	if (progress) {
		result = progress;
		*ppos += progress;
	}

out_free:
	kfree(iobuf);
	return result;
}

/*
 * Check the conditions on a file descriptor prior to beginning a write
 * on it.  See 2.4.22's mm/filemap.c:precheck_file_write
 *
 * In the direct I/O case, can we simply let the server handle
 * bounds checking?  seems that might close some race windows.
 */
static int
nfs_precheck_file_write(struct file *file, struct inode *inode,
	size_t *count, loff_t *offset)
{
	ssize_t		err;
	unsigned long	limit = current->rlim[RLIMIT_FSIZE].rlim_cur;
	loff_t		pos = *offset;

	err = -EINVAL;
	if (pos < 0)
		goto out;

	err = file->f_error;
	if (err) {
		file->f_error = 0;
		goto out;
	}

#if 0
	if (file->f_flags & O_APPEND)
		*offset = pos = inode->i_size;

	/*
	 * Check whether we've reached the file size limit.
	 */
	err = -EFBIG;
	if (limit != RLIM_INFINITY) {
		if (pos >= limit) {
			send_sig(SIGXFSZ, current, 0);
			goto out;
		}
		if (pos > 0xFFFFFFFFULL || *count > limit - (u32)pos)
			*count = limit - (u32)pos;
	}

	/*
	 *	LFS rule
	 */
	if (pos + *count > MAX_NON_LFS && !(file->f_flags&O_LARGEFILE)) {
		if (pos >= MAX_NON_LFS) {
			send_sig(SIGXFSZ, current, 0);
			goto out;
		}
		if (*count > MAX_NON_LFS - (u32)pos)
			*count = MAX_NON_LFS - (u32)pos;
	}

	/*
	 *	Are we about to exceed the fs block limit ?
	 *
	 *	If we have written data it becomes a short write
	 *	If we have exceeded without writing data we send
	 *	a signal and give them an EFBIG.
	 */
	if (pos >= inode->i_sb->s_maxbytes) {
		if (*count || pos > inode->i_sb->s_maxbytes) {
			send_sig(SIGXFSZ, current, 0);
			err = -EFBIG;
			goto out;
		}
		/* zero-length writes at ->s_maxbytes are OK */
	}

	if (pos + *count > inode->i_sb->s_maxbytes)
		*count = inode->i_sb->s_maxbytes - pos;

	err = 0;
#endif
out:
	return err;
}

static int
nfs_remove_suid(struct inode * inode)
{
	int error = 0;
	struct nfs_fattr fattr;
	struct iattr attr;

	memset(&attr, 0, sizeof(struct iattr));

	/* set S_IGID if S_IXGRP is set, and always set S_ISUID */
	attr.ia_mode = (inode->i_mode & S_IXGRP)*(S_ISGID/S_IXGRP) | S_ISUID;

	/* were any of the uid bits set? */
	attr.ia_mode &= inode->i_mode;
	if (attr.ia_mode && !capable(CAP_FSETID)) {
		inode->i_mode &= ~attr.ia_mode;
		printk(KERN_ERR "%s: resetting SUID bits\n", __FUNCTION__);
		error = NFS_PROTO(inode)->setattr(inode, &fattr, &attr);
		if (!error)
			error = nfs_refresh_inode(inode, &fattr);
	}

	return error;
}

ssize_t
nfs_direct_file_write(struct file *file, const char *buf, size_t count,
		loff_t *ppos)
{
	ssize_t result;
	int iosize, progress;
	loff_t offset = *ppos;
	struct dentry *dentry = file->f_dentry;
	struct inode *inode = dentry->d_inode;
	struct kiobuf_nfs_direct *iobuf;

	result = nfs_precheck_file_write(file, inode, &count, &offset);
	if (result)
		return result;

	iobuf = kmalloc(sizeof(struct kiobuf_nfs_direct), GFP_KERNEL);
	if (!iobuf)
		return -ENOMEM;
	memset(iobuf, 0, sizeof(struct kiobuf_nfs_direct));

	down(&inode->i_sem);

	if (file->f_flags & O_APPEND) {
		offset = inode->i_size;
		*ppos = offset;
	}
		

	/* start any latent normal read requests now to maintain
	 * request ordering */
	nfs_pagein_inode(inode, 0, 0);

	filemap_fdatasync(inode->i_mapping);
	result = nfs_wb_all(inode);
	filemap_fdatawait(inode->i_mapping);
	if (result < 0)
		goto out;

	result = 0;
	if (nfs_remove_suid(inode))
		goto out;

	dfprintk(VFS, "NFS: direct_IO(WRITE) (%s/%s) off/cnt(%Lu/%d)\n",
			dentry->d_parent->d_name.name, dentry->d_name.name,
			offset, count);

	progress = 0;
	while (count > 0) {
		iosize = count;
		if (iosize > NFS_KIOBUF_SIZE)
			iosize = NFS_KIOBUF_SIZE;

		result = map_user_kiobuf(WRITE, (struct kiobuf *)iobuf, (unsigned long) buf,
				iosize);
		if (result)
			break;

		result = nfs_kiobuf_write(file, iobuf, offset + progress, count);
		if (result > 0) {
			count -= result;
			buf += result;
			progress += result;
		}

		unmap_kiobuf(iobuf);
		kfree(iobuf->maplist);

		if (result != iosize)
			break;
	}

	if (progress) {
		result = progress;
		*ppos += progress;
	}

	/* local non-O_DIRECT accessors need to see these changes */
	invalidate_inode_pages(inode);

out:
	up(&inode->i_sem);
	kfree(iobuf);
	return result;
}
